/* Copyright (c) 2004-2006 ActiveState Software Inc.
   See the file LICENSE.txt for licensing information. */

//---- globals

var gDOMViewMgr = null;
var gDOMViewCtrl = null;
var _gDOMView_log = null;
var _gDOMView_observer = null;



//---- observer

function _DOMViewObserver()
{
    try {
        window.addEventListener('current_view_changed',
                                this.handle_common_view_change, false);
        window.addEventListener('current_view_language_changed',
                                this.handle_common_view_change, false);
        window.addEventListener('view_closed',
                                this.handle_common_view_change, false);
        window.addEventListener('view_opened',
                                this.handle_common_view_change, false);
        window.addEventListener('view_document_attached',
                                this.handle_view_document_attached, false);
        window.addEventListener('view_document_detaching',
                                this.handle_view_document_detaching, false);
        this._view = null;
    } catch(ex) {
        _gDOMView_log.exception(ex);
    }
}
_DOMViewObserver.prototype.constructor = _DOMViewObserver;

_DOMViewObserver.prototype.finalize = function()
{
    try {
        window.removeEventListener('current_view_changed',
                                   this.handle_common_view_change, false);
        window.removeEventListener('current_view_language_changed',
                                   this.handle_common_view_change, false);
        window.removeEventListener('view_closed',
                                   this.handle_common_view_change, false);
        window.removeEventListener('view_opened',
                                   this.handle_common_view_change, false);
        window.removeEventListener('view_document_attached',
                                   this.handle_view_document_attached, false);
        window.removeEventListener('view_document_detaching',
                                   this.handle_view_document_detaching, false);
    } catch(ex) {
        _gDOMView_log.exception(ex);
    }
}

_DOMViewObserver.prototype.observe = function(subject, topic, data) {
    if (topic == "DOMViewer_enabled") {
        gDOMViewMgr.view_changed(ko.views.manager.currentView);
    }
}

_DOMViewObserver.prototype.handle_view_document_attached = function(event)
{
    var view = event.originalTarget;
    if (view.getAttribute("type") != "editor") {
        return;
    }
    view.prefs.prefObserverService.addObserver(_gDOMView_observer, "DOMViewer_enabled", false);
}

_DOMViewObserver.prototype.handle_view_document_detaching = function(event)
{
    var view = event.originalTarget;
    if (view.getAttribute("type") != "editor") {
        return;
    }
    view.prefs.prefObserverService.removeObserver(_gDOMView_observer, "DOMViewer_enabled", false);
}

_DOMViewObserver.prototype.handle_common_view_change = function(event)
{
    if (ko.views.manager.batchMode) {
        return;
    }
    gDOMViewMgr.view_changed(event.originalTarget);
};

//---- the UI Manager for the DOM View

function DOMViewManager(treeId) {
    try {
        this.dom = document.getElementById('domview-tree');
        this.filter = document.getElementById('domview-filter-textbox');
        this._resetTimeout = null;
        var ciSvc = Components.classes["@activestate.com/koCodeIntelService;1"].
              getService(Components.interfaces.koICodeIntelService);
        if (ciSvc.enabled) {
            this.Activate();
        }
        this.isDisabledForCurrentView = false;
    } catch (e) {
        _gDOMView_log.exception(e);
    }
}
DOMViewManager.prototype.constructor = DOMViewManager;

DOMViewManager.prototype.__defineGetter__("view",
function()
{
    if (!this.dom) return null;
    return this.dom.view;
});

DOMViewManager.prototype.focus = function() {
    _gDOMView_log.debug("DOMViewManager.focus()");
    try {
        // Pass the focus onto the <tree/>.
        // It is important to do this because this focus handling is used to
        // get uilayout_toggleTab() to work properly.
        this.dom.focus();
    } catch(ex) {
        _gDOMView_log.exception(ex);
    }
}

DOMViewManager.prototype.toggleDetail = function(view) {
    _gDOMView_log.debug("DOMViewManager.toggleDetail()");
    try {
        this.dom.view.showDetails = ! this.dom.view.showDetails;
    } catch(ex) {
        _gDOMView_log.exception(ex);
    }

}


DOMViewManager.prototype.updateFilter = function() {
    _gDOMView_log.debug("DOMViewManager.updateFilter()");
    try {
        var filterString = this.filter.value;
        this.dom.view.filterString = filterString;
    } catch(ex) {
        _gDOMView_log.exception(ex);
    }
}

DOMViewManager.prototype._supportedDomLanguages = ["XML", "HTML", "HTML5", "XHTML", "XSLT"];

DOMViewManager.prototype.isSupportedDomLanguage = function(view) {
    var langSvc = view.koDoc.languageObj;
    if (langSvc.isHTMLLanguage) {
        return true;
    }
    var domLanguages = this._supportedDomLanguages;
    var found = domLanguages.indexOf(langSvc.name) >= 0;
    if (!found) {
        var languages = langSvc.getSubLanguages(new Object());
        for (var i=0; i < languages.length; i++) {
            if (domLanguages.indexOf(languages[i]) >= 0) {
                found = true;
                break;
            }
        }
    }
    if (!found) {
        return false;
    }
    return true;
}

DOMViewManager.prototype.isEnabledForView = function(view) {
    if (!view.koDoc)
        return false;
    var viewPrefs = view.prefs;
    if (viewPrefs.hasPrefHere("DOMViewer_enabled")
        && !viewPrefs.getBooleanPref("DOMViewer_enabled")) {
        _gDOMView_log.debug("DOMViewer is disabled through prefs");
        return false;
    }
    return true;
}

DOMViewManager.prototype.getDOMView = function(view) {
    if (!view.domView) {
        view.domView = Components.classes['@activestate.com/KoDOMTreeView;1'].
                            createInstance(Components.interfaces.koIDOMTreeView);
        view.domView.setDocument(view.koDoc);
        view.domView.reset();
    }
    return view.domView;
}

DOMViewManager.prototype.isVisible = function(view) {
    try {
        // Find the parent tabbox;
        var tabbox = this.dom;
        while (tabbox && tabbox.localName != "tabbox") {
            tabbox = tabbox.parentNode;
        }
        if (!tabbox) {
            _gDOMView_log.error("Unable to find the dom's tabbox");
            return true;
        }
        if (tabbox.parentNode.getAttribute("collapsed") == "true") {
            // The side-pane is not visible.
            return false;
        }
        if (tabbox.selectedTab.getAttribute('id') != 'domviewer_tab') {
            // It's not the selected tab.
            return false;
        }
    } catch(ex) {
        _gDOMView_log.exception(ex);
    }
    return true;
}

DOMViewManager.prototype.updateDOMView = function(view) {
    _gDOMView_log.debug("DOMViewManager.updateDOMView()");
    try {
        if (this.isDisabledForCurrentView) {
            _gDOMView_log.info("DOMViewManager.updateDOMView():: disabled for this view");
            return;
        }
        if (!this.isVisible()) {
            _gDOMView_log.info("DOMViewManager.updateDOMView():: not visible");
            // We'll update it when if becomes visible again.
            // Note: updateDOMView will be called when it's made visible (due to
            //       oncommand, onfocus handlers in the overlay).
            return;
        }

        if (typeof(view) == 'undefined')
            view = ko.views.manager.currentView;
        var domView = this.getDOMView(view);
        /* XXX breaks switching documents in dom viewer
        if (domView) {
            if (domView.selection && !domView.selection.count) {
                // there is no selection, impose one.
                domView.selection.select(0);
            }
        }
        */ 
        this.dom.treeBoxObject
                        .QueryInterface(Components.interfaces.nsITreeBoxObject)
                        .view = domView;
    } catch(ex) {
        _gDOMView_log.exception(ex);
    }
}

DOMViewManager.prototype.onModifiedHandler = function(position,modificationType,text,length,linesAdded,line,foldLevelNow,foldLevelPrev)
{
    if (!this.isVisible()) {
        return false; // let onmodified handler continue
    }

    var view = ko.views.manager.currentView;
    // if deleting or inserting text...
    if (view.domView)
    {
        if (this._resetTimeout) {
            window.clearTimeout(this._resetTimeout);
        }
        var self = this;
        this._resetTimeout = window.setTimeout(
                            function() {
                                view.domView.reset();
                                self._resetTimeout = null;
                            }, 500)
    }
    return false; // let onmodified handler continue
}

DOMViewManager.prototype.disableViewer = function(view)
{
    if (this.isDisabledForCurrentView) {
        // Already disabled
        return;
    }
    this.isDisabledForCurrentView = true;
    // Null out the current dom.
    view.domView = null;
    this.dom.treeBoxObject
                    .QueryInterface(Components.interfaces.nsITreeBoxObject)
                    .view = null;
}

DOMViewManager.prototype.view_changed = function(view)
{
    try {
        if (!this.isEnabledForView(view)) {
            // Nothing to do if the user has turned it off.
            this.disableViewer(view);
            return;
        }
        if (!this.isSupportedDomLanguage(view)) {
            // Nothing to do if the language is not supported.
            this.disableViewer(view);
            return;
        }
        this.isDisabledForCurrentView = false;

        if (this._view && this._view.removeModifiedHandler) {
            this._view.removeModifiedHandler(this.onModifiedHandler);
        }
        if (!view) return;
        this._view = view; // Keep view around so we can remove our handler later on.
        var wantedFlags = (Components.interfaces.ISciMoz.SC_MOD_INSERTTEXT |
                           Components.interfaces.ISciMoz.SC_MOD_DELETETEXT);
        view.addModifiedHandler(this.onModifiedHandler, this, 90, wantedFlags);
    
        this.updateDOMView(view);
    } catch(ex) {
        _gDOMView_log.exception(ex);
    }
}

DOMViewManager.prototype._jumpToNode = function(gotoEnd) {
    _gDOMView_log.debug("DOMViewManager._jumpToNode()");
    try {
        var currentIndex = this.dom.view.selection.currentIndex;
        if (currentIndex == -1) {
            return;
        }
        // DOMView is always for the current view
        var view = ko.views.manager.currentView;
        var start = new Object();
        this.dom.view.getStartPos(currentIndex, start, new Object());
        start = start.value;
        var startPos = view.scimoz.positionFromLine(start[0]-1) + start[1];
        var endPos = startPos;

        var end = new Object();
        this.dom.view.getEndPos(currentIndex, end, new Object());
        end = end.value;
        if (end.length > 0) {
            endPos = view.scimoz.positionFromLine(end[0]-1) + end[1];
        }

        if (gotoEnd || view.scimoz.currentPos == startPos) {
            view.scimoz.gotoPos(endPos);
        } else {
            view.scimoz.gotoPos(startPos);
        }
        view.setFocus();
    } catch(ex) {
        _gDOMView_log.exception(ex);
    }
}

DOMViewManager.prototype._selectNode = function() {
    _gDOMView_log.debug("DOMViewManager._selectNode()");
    try {
        var currentIndex = this.dom.view.selection.currentIndex;
        if (currentIndex == -1) {
            return;
        }
        // DOMView is always for the current view
        var view = ko.views.manager.currentView;
        var start = new Object();
        this.dom.view.getStartPos(currentIndex, start, new Object());
        start = start.value;
        var startPos = view.scimoz.positionFromLine(start[0]-1) + start[1];

        var end = new Object();
        this.dom.view.getEndPos(currentIndex, end, new Object());
        end = end.value;
        var endPos = view.scimoz.positionFromLine(end[0]-1) + end[1];

        view.scimoz.anchor = startPos;
        view.scimoz.currentPos = endPos;
        view.scimoz.scrollCaret();
        view.setFocus();
    } catch(ex) {
        _gDOMView_log.exception(ex);
    }
}

DOMViewManager.prototype.onTreeKeyPress = function(event) {
    _gDOMView_log.debug("DOMViewManager.onTreeKeyPress(event)");
    try {
        if (event.keyCode == event.DOM_VK_ENTER
            || event.keyCode == event.DOM_VK_RETURN)
        {
            event.cancelBubble = true;
            event.preventDefault();
            if (event.shiftKey) {
                window.setTimeout("gDOMViewMgr._selectNode();", 0);
            } else if (event.ctrlKey) {
                window.setTimeout("gDOMViewMgr._jumpToNode(1);", 0);
            } else {
                window.setTimeout("gDOMViewMgr._jumpToNode(0);", 0);
            }
            return;
        }
        if (event.keyCode == event.DOM_VK_ESCAPE)
        {
            event.cancelBubble = true;
            event.stopPropagation();
            event.preventDefault();
            this.filter.value = '';
            this.updateFilter();
            return;
        }
        if (event.keyCode == event.DOM_VK_TAB && !event.ctrlKey)
        {
            event.cancelBubble = true;
            event.stopPropagation();
            event.preventDefault();
            window.setTimeout("gDOMViewMgr.focusFilterBox();", 0);
        }
    } catch(ex) {
        _gDOMView_log.exception(ex);
    }
}


DOMViewManager.prototype.focusFilterBox = function(event) {
    this.filter.focus();
}

DOMViewManager.prototype.onFilterKeypress = function(event) {
    _gDOMView_log.debug("DOMViewManager.onTreeKeyPress(event)");
    try {
        if (event.keyCode == event.DOM_VK_TAB && !event.ctrlKey)
        {
            this.dom.focus();
        }
        if (event.keyCode == event.DOM_VK_ESCAPE)
        {
            event.cancelBubble = true;
            event.stopPropagation();
            event.preventDefault();
            this.filter.value = '';
            this.updateFilter();
            this.dom.focus();
            return;
        }
    } catch(ex) {
        _gDOMView_log.exception(ex);
    }
}

DOMViewManager.prototype.onDblClick = function(event) {
    _gDOMView_log.debug("DOMViewManager.onDblClick(event)");
    try {
        if (event.shiftKey) {
            this._selectNode();
        } else {
            this._jumpToNode(0);
        }
    } catch(ex) {
        _gDOMView_log.exception(ex);
    }
}

// Set a new sort order for the code browser.
// Returns true iff successful.
DOMViewManager.prototype.sortBy = function(key) {
    _gDOMView_log.debug("DOMViewManager.sortBy(key='"+key+"')");
    try {
    } catch(ex) {
        _gDOMView_log.exception(ex);
    }
}

DOMViewManager.prototype.Activate = function() {
    window.controllers.insertControllerAt(0, gDOMViewCtrl);
    // Show the Code Browser tab.
    var tab = document.getElementById("domviewer_tab");
    if (tab.hasAttribute("collapsed"))
        tab.removeAttribute("collapsed");
    // <tabbox> Ctrl+Tab handling uses 'hidden' attribute.
    if (tab.hasAttribute("hidden"))
        tab.removeAttribute("hidden");
}

DOMViewManager.prototype.Deactivate = function() {
    // Hide the Code Browser tab.
    var tabbox = document.getElementById("leftTabBox");
    var tab = document.getElementById("domviewer_tab");
    if (tabbox.selectedTab == tab) {
        var alternate = tab.previousSibling;
        if (!alternate)
            alternate = tab.nextSibling;
        if (alternate)
            tabbox.selectedTab = alternate;
    }
    tab.setAttribute("collapsed", "true");
    // <tabbox> Ctrl+Tab handling uses 'hidden' attribute.
    tab.setAttribute("hidden", "true");
    window.controllers.removeController(gDOMViewCtrl);
}

//---- the controller for the code browser commands

function DOMViewController() {
}
// The following two lines ensure proper inheritance (see Flanagan, p. 144).
DOMViewController.prototype = new xtk.Controller();
DOMViewController.prototype.constructor = DOMViewController;

DOMViewController.prototype.destructor = function() {
}

DOMViewController.prototype.is_cmd_documentOutlineJumpToNode_enabled = function() {
    return gDOMViewMgr && gDOMViewMgr.view.selection.count != 0;
}

DOMViewController.prototype.do_cmd_documentOutlineJumpToNode = function() {
    gDOMViewMgr._jumpToNode(0);
}

DOMViewController.prototype.is_cmd_documentOutlineSelectNode_enabled = function() {
    return gDOMViewMgr && gDOMViewMgr.view.selection.count != 0;
}

DOMViewController.prototype.do_cmd_documentOutlineSelectNode = function() {
    gDOMViewMgr._selectNode();
}

DOMViewController.prototype.is_cmd_documentOutlineLocateCurrentNode_enabled = function() {
    if (!gDOMViewMgr) return false;
    var view = ko.views.manager.currentView;
    return (view && view.scimoz && view.koDoc && view.koDoc.file);
}

DOMViewController.prototype.do_cmd_documentOutlineLocateCurrentNode = function() {
    var currentLine, currentCol;
    if (!ko.views.manager || !ko.views.manager.currentView || !ko.views.manager.currentView.scimoz) {
        return;
    }
    if (view_elementHasFocus(ko.views.manager.currentView)) {
        // If the focus is in the editor, select the node in the DOM viewer which clos
        var scimoz = ko.views.manager.currentView.scimoz;
        var currentPos = scimoz.currentPos;
        currentLine = scimoz.lineFromPosition(currentPos);
        currentCol = scimoz.getColumn(currentPos);
        gDOMViewMgr.view.QueryInterface(Components.interfaces.koIDOMTreeView).locateNode(currentLine, currentCol);
    } else {
        gDOMViewMgr._jumpToNode(0);
    }
}

DOMViewController.prototype.is_cmd_documentViewerToggleDetail_enabled = function() {
    return gDOMViewMgr != null;
}
DOMViewController.prototype.do_cmd_documentViewerToggleDetail = function() {
    gDOMViewMgr.toggleDetail();
}
//---- public routines

function DOMView_Initialize()
{
    _gDOMView_log = getLoggingMgr().getLogger("domview_js");
    //_gDOMView_log.setLevel(LOG_DEBUG);
    _gDOMView_log.debug("DOMView_Initialize()");
    try {
        _gDOMView_observer = new _DOMViewObserver();
        gDOMViewCtrl = new DOMViewController();
        gDOMViewMgr = new DOMViewManager();
    } catch(ex) {
        _gDOMView_log.exception(ex);
    }
    window.removeEventListener("load", DOMView_Initialize, false);
}

function DOMView_Finalize()
{
    _gDOMView_log.debug("DOMView_Finalize()");
    try {
        _gDOMView_observer.finalize();
    } catch(ex) {
        _gDOMView_log.exception(ex);
    }
}

